<?php

namespace Drupal\commerce_gpwebpay\PluginForm;

use Drupal\commerce_payment\Exception\InvalidRequestException;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use Chaching\Currencies;
use Chaching\Driver;

/**
 * PaymentGPWebpayForm class.
 */
class PaymentGPWebpayForm extends BasePaymentOffsiteForm {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;

    /** @var \Drupal\commerce_gpwebpay\Plugin\Commerce\PaymentGateway\GPWebpay $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $chaching = $payment_gateway_plugin->getDriverInstance();

    $order = $payment->getOrder();

    $gp_currency = Currencies::get($order->getTotalPrice()->getCurrencyCode());
    if (empty($gp_currency)) {
      throw new InvalidRequestException("Currency code is not supported");
    }

    $payment->setRemoteId($this->getRemoteId($order->id()));
    $gp_payment = $chaching->request([
      Driver::CURRENCY => $gp_currency['numeric_code'],
      // This cannot be getOrderNumber() as this can be string.
      // GP Webpay supports only numeric dat for this field.
      Driver::VARIABLE_SYMBOL => (int) $payment->getRemoteId(),
      Driver::AMOUNT => $order->getTotalPrice()->getNumber(),
      // @todo Make description configurable with tokens. GP does not accept non-ascii characters in order description.
      Driver::DESCRIPTION => '',
      Driver::CALLBACK => $form['#return_url'],
      'MERORDERNUM' => $order->id(),
      // @todo Add `ADDINFO` field that describes cart contents.
      // @todo Add `EMAIL` field.
    ]);

    $execute_redirection = FALSE;
    $redirect_url = $gp_payment->process($execute_redirection);

    return $this->buildRedirectForm(
      $form, $form_state, $redirect_url, [], self::REDIRECT_GET);
  }

  /**
   * Calculate Remote ID.
   *
   * We are concatenating order ID and timestamp of the payment. We will first
   * cut down order ID down to 5 digits and in the end we will cut down whole
   * string down to 15 digits.
   *
   * @param $orderId
   *   The ID of the order.
   *
   * @return int
   *   Generated Remote ID. This ID can be used for tracking down the payment.
   */
  private function getRemoteId($orderId): int {
    // Cut Order ID down to 5 characters.
    $shortenOrderId = str_pad(substr($orderId, -5), 5, 0, STR_PAD_LEFT);
    $timestamp = time();
    $remoteId = $timestamp . $shortenOrderId;
    // Cut down concatenated Remote ID to 15 chars.
    $remoteId = substr($remoteId, -15);
    return (int) $remoteId;
  }

}
